package fr.iut.valence.java.M2M;

import java.io.Serializable;
import java.util.Date;

/**
 * @author FAURE Adrien, GRAILLAT Amaury
 */
public class Message implements Serializable
{
	/**
	 * Version de la serialisation 
	 */
	private static final long serialVersionUID = -8148148918931881537L;
	
	private final String destinataire;
	private final String source;
	private final String message;
	private final Date date;
	
	/**
	 * Constructeur
	 * @author FAURE Adrien, GRAILLAT Amaury
	 * @param dest : Pseudonyme du destinataire
	 * @param src : Pseudonyme de l'émetteur
	 * @param msg : Le texte du message
	 * @param dateMsg : Date d'envoie du message
	 */
	public Message(String dest, String src, String msg, Date dateMsg)
	{
		this.message = msg;
		this.destinataire = dest;
		this.source = src;
		this.date = dateMsg;
	}
	
	/**
	 * Constructeur
	 * @author FAURE Adrien, GRAILLAT Amaury
	 * @param dest :  Pseudonyme du destinataire
	 * @param src : Pseudonyme de l'émetteur
	 * @param msg : Le texte du message
	 */
	public Message(String dest, String src, String msg)
	{
		this.message = msg;
		this.destinataire = dest;
		this.source = src;
		this.date = new Date();
	}
	
	public String toString()
	{
		return this.getSource() + " > " + this.getDestinataire() + " (" + this.date + ")" + " : " + this.getTexte() + "\n";
	}
	
	/**
	 * Renvoie le contenu du message
	 * @author FAURE Adrien, GRAILLAT Amaury
	 * @return String : Le texte
	 */
	public String getTexte()
	{
		return this.message;
	}
	
	/**
	 * Renvoie l'émetteur du messages
	 * @author FAURE Adrien, GRAILLAT Amaury
	 * @return String : Le pseudo
	 */
	public String getSource()
	{
		return this.source;
	}
	
	/**
	 * Renvoie le destinataire du message
	 * @author FAURE Adrien, GRAILLAT Amaury
	 * @return String : Le pseudo
	 */
	public String getDestinataire()
	{
		return this.destinataire;
	}
	
	/**
	 * Renvoie la date d'envoie du message
	 * @author FAURE Adrien, GRAILLAT Amaury
	 * @return Date : date d'envoie
	 */
	public Date getDate()
	{
		return this.date;
	}
	
}
