package fr.iut.valence.java.M2M;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class SyntaxeV1 implements Syntaxe
{
	/**
	 * Création du message en fonction de la commande reçue
	 * @param String Commande reçue
	 * @return Message ou null
	 */
	public Message getMessage(String ligne)
	{
		// Commande de type message
		Pattern p=Pattern.compile("^MSG (.*) FROM=(.*) TO=(.*);$");
		Matcher m=p.matcher(ligne);
		Message mess = null;
		if (m.find())
		{
			// Extraction dans un message
			mess = new Message(m.group(3), m.group(2), m.group(1));
		}
		return mess; 
	}
	
	/**
	 * Création de la commande correspondant à un Message
	 * @param Message m
	 * @return String la commande
	 */
	public String cmdMessage(Message m)
	{
		return "MSG " + m.getTexte() + " FROM=" + m.getSource() + " TO=" + m.getDestinataire() + ";";
	}
	
	/**
	 * Récupère le pseudo du client
	 * @param ligne ligne
	 * @return String Pseudo ou null
	 */
	public String getPseudo(String ligne)
	{
		Pattern p=Pattern.compile("^EHLO (.*);$");
		Matcher m=p.matcher(ligne);
		String pseudo = null;
		if (m.find())
		{
			pseudo = m.group(1);
		}
		return pseudo;
	}

	/**
	 * @param Message un message
	 * @return String commande correspondante
	 */
	public String cmdPseudo(String p)
	{
		return "EHLO " + p + ";";
	}

	/**
	 * Récupère le message d'erreur
	 * @param ligne ligne
	 * @return String erreur ou null
	 */
	public Erreur getErreur(String ligne)
	{
		Pattern p=Pattern.compile("^ERROR ID=([0-9]+) MSG=(.*);$");
		Matcher m=p.matcher(ligne);
		
		if (m.find())
		{
			int id = Integer.parseInt(m.group(1));
			String msg = m.group(2);
			return new Erreur(id, msg);
		}
		return null;
	}

	/**
	 * @param String une erreur
	 * @return String commande correspondante
	 */
	public String cmdErreur(Erreur e)
	{
		return "ERROR ID=" + e.getId() + " MSG=" + e.getMessage() + ";";
	}
}
