package fr.iut.valence.java.M2M.client;

import java.io.IOException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import fr.iut.valence.java.M2M.Erreur;
import fr.iut.valence.java.M2M.Message;
import fr.iut.valence.java.M2M.Recepteur;

public abstract class AbstractCommunicationClient implements Recepteur
{
	/* List contenant les interfaces de sortie */
	private Set<TraitementMessages> interfacesTraitements;
	
	
	public AbstractCommunicationClient()
	{
		this.interfacesTraitements = new HashSet<TraitementMessages>();
	}
	
	/**
	 * Ajout d'un objet de traitement des messages
	 * @param t objet de traitement
	 */
	public void ajoutObjetTraitement(TraitementMessages t)
	{
		this.interfacesTraitements.add(t);
	}
	
	/**
	 * Suppression d'un objet de traitement des messages
	 * @param t objet de traitement
	 */
	public void suppressionObjetTraitement(TraitementMessages t)
	{
		this.interfacesTraitements.remove(t);
	}
	
	/**
	 * Réception d'un nouveau message
	 * @param msg
	 */
	public void receptionMessage(Message msg)
	{
		Iterator<TraitementMessages> i = this.interfacesTraitements.iterator();
		
		// Passage des messages aux interfaces
		while(i.hasNext())
		{
			TraitementMessages t = i.next();
			t.traiteMessage(msg);
		}
	}
	

	/**
	 * Réception d'une erreur
	 * @param Message Message à envoyer
	 * @throws IOException
	 */
	public void receptionErreur(Erreur erreur)
	{
		Iterator<TraitementMessages> i = this.interfacesTraitements.iterator();
		
		// Passage des messages aux interfaces
		while(i.hasNext())
		{
			TraitementMessages t = i.next();
			t.traiteErreur(erreur);
		}
	}
}
