package fr.iut.valence.java.M2M.client;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import fr.iut.valence.java.M2M.Erreur;
import fr.iut.valence.java.M2M.Message;
import fr.iut.valence.java.M2M.SyntaxeV1;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.InetSocketAddress;
import java.util.Map;
import java.util.TreeMap;

public class FenPrincipaleApplicationGraphique implements ActionListener, TraitementMessages, ChangeListener, MouseListener
{
	private ApplicationClient appClient;
	private JButton btn_connect;
	private JButton btn_parler;
	private InfoClient iClient;
	private JTextField edit_pseudo;
	private JTextField edit_destpseudo;
	private InetSocketAddress serveur;
	private SocketCommunicationClient socket;
	private SyntaxeV1 syntaxe;
	private JTextField edit_port;
	private JTextField edit_serveur;

	private JFrame fenetrePrincipale;

	// Objet de stockage des fenêtres ouvertes
	private Map<String, OngletConversationApplicationGraphique> conversations;
	private JPanel panelConnexion;
	private JPanel panelConversation;
	private JTabbedPane onglets;
	private JPanel ongletConversation;
	
	/**
	 * Constructeur de la fenêtre pricipale
	 * 
	 * @throws HeadlessException
	 */
	public FenPrincipaleApplicationGraphique() throws HeadlessException
	{
		// Initialisation de la liste des fenêtres ouvertes
		this.conversations = new TreeMap<String, OngletConversationApplicationGraphique>();
		this.syntaxe = new SyntaxeV1();

		// Partie connexion
		this.panelConnexion = new JPanel();
		this.edit_pseudo = new JTextField("Pseudo", 8);
		this.edit_serveur = new JTextField("127.0.0.1", 8);
		this.edit_port = new JTextField("1111", 4);
		this.btn_connect = new JButton("Connexion");
		this.panelConnexion.add(this.edit_serveur);
		this.panelConnexion.add(this.edit_port);
		this.panelConnexion.add(this.edit_pseudo);
		this.panelConnexion.add(this.btn_connect);

		// Partie ouverture de conversation
		this.panelConversation = new JPanel();
		this.panelConversation.setLayout(new BorderLayout());
		this.onglets = new JTabbedPane();
		this.panelConversation.add(this.onglets, BorderLayout.CENTER);

		this.ongletConversation = new JPanel();
		this.edit_destpseudo = new JTextField("Pseudo");
		this.btn_parler = new JButton("Ouvrir la conversation");
		this.ongletConversation.add(this.edit_destpseudo);
		this.ongletConversation.add(this.btn_parler);
		this.onglets.addTab("Contacts", this.ongletConversation);

		// Gestion des événements :
		this.btn_parler.addActionListener(this);
		this.btn_connect.addActionListener(this);
		this.onglets.addChangeListener(this);
		this.onglets.addMouseListener(this);
		
		// Initialisation de la fenêtre
		this.fenetrePrincipale = new JFrame("M2M powered");
		this.fenetrePrincipale.setContentPane(this.panelConnexion);
		this.fenetrePrincipale.setSize(640, 480);
		this.fenetrePrincipale.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.fenetrePrincipale.setVisible(true);
	}

	/**
	 * Gestion des événements
	 */
	public void actionPerformed(ActionEvent e)
	{	
		// Bouton d'ouverture d'une conversation
		if (e.getSource() == this.btn_parler)
		{
			// La conversation existe déjà
			if (this.conversations.get(this.edit_destpseudo.getText()) != null)
			{
				this.conversations.get(this.edit_destpseudo.getText()).setVisible(true);
				this.onglets.setSelectedIndex(this.onglets.indexOfTab(this.edit_destpseudo.getText()));
			}
			// On crée une nouvelle fenêtre de conversation
			else
			{
				OngletConversationApplicationGraphique fen = new OngletConversationApplicationGraphique(
						this.socket, this.edit_destpseudo.getText(),this.edit_pseudo.getText());
				
				this.onglets.addTab(this.edit_destpseudo.getText(), fen);
				this.conversations.put(this.edit_destpseudo.getText(), fen);
				this.onglets.setSelectedIndex(this.onglets.indexOfTab(this.edit_destpseudo.getText()));
			}
		}
		// Bouton de connexion
		else if (e.getSource() == this.btn_connect)
		{
			this.iClient = new InfoClient(this.edit_pseudo.getText());
			this.serveur = new InetSocketAddress(this.edit_serveur.getText(),
					Integer.parseInt(this.edit_port.getText()));

			try
			{
				this.socket = new SocketCommunicationClient(this.serveur,
					this.syntaxe, this.iClient);

				// Autorise l'ouverture de conversations
				this.fenetrePrincipale.setVisible(false);
				this.fenetrePrincipale.setContentPane(this.panelConversation);
				this.fenetrePrincipale.setVisible(true);

				// Ouverture de la connexion avec le serveur
				this.appClient = new ApplicationClient(this.socket);
				this.socket.setAppClient(this.appClient);
				this.appClient.ajoutObjetTraitement(this);
				this.socket.start();

				this.fenetrePrincipale.setSize(640, 480);
			}
			// Erreur de connexion
			catch (Exception e1)
			{
				this.traiteErreur(new Erreur(Erreur.CONNEXION,
						"Serveur injoignable, vérifiez l'adresse."));
			}
		}
	}

	/**
	 * Réception et affichage d'un nouveau message
	 * 
	 * @param Message
	 *            un message
	 */
	public void traiteMessage(Message m)
	{
		// La conversation existe déjà
		if (this.conversations.get(m.getSource()) != null)
		{
			this.conversations.get(m.getSource()).affMessage(m);
		}
		// On crée une nouvelle fenêtre de conversation
		else
		{
			OngletConversationApplicationGraphique fen = new OngletConversationApplicationGraphique(
					this.socket, m.getSource(), this.edit_pseudo.getText());
			this.conversations.put(m.getSource(), fen);
			fen.affMessage(m);
			this.onglets.addTab(m.getSource(), fen);
		}
		
		// Informe l'utilisateur de l'arrivée du message (s'il n'est pas déjà l'onglet)
		this.setNotificationOnglet(m.getSource(), true);
	}

	/**
	 * Met en valeur un onglet
	 * @param pseudo : pseudo du contact
	 * @param notifier : indique si l'onglet est mis en valeur 
	 */
	private void setNotificationOnglet(String pseudo, boolean notifier)
	{
		if(this.conversations.containsKey(pseudo))
		{
			if(notifier)
			{
				this.fenetrePrincipale.setTitle("Nouveau message - M2M powered");
				this.onglets.setBackgroundAt(this.onglets.indexOfTab(pseudo), Color.RED);
			}
			else
			{
				this.fenetrePrincipale.setTitle("M2M powered");
				this.onglets.setBackgroundAt(this.onglets.indexOfTab(pseudo), Color.WHITE);
			}
		}
	}

	/**
	 * Réception d'une erreur
	 * 
	 * @param String
	 *            err
	 */
	public void traiteErreur(Erreur err)
	{
		JOptionPane.showMessageDialog(null, "Erreur : " + err.getMessage());

		// Connexion écouée
		if (err.getId() == Erreur.CONNEXION	|| err.getId() == Erreur.PSEUDO_DEJA_UTILISE)
		{
			this.fenetrePrincipale.setContentPane(this.panelConnexion);
		}
	}

	/**
	 * Évenements sur le changement d'onglet
	 */
	public void stateChanged(ChangeEvent e)
	{
		setNotificationOnglet(this.onglets.getTitleAt(this.onglets.getSelectedIndex()), false);  
	}

	/**
	 * Événements au clic
	 */
	public void mouseClicked(MouseEvent e)
	{
		// Ferme l'onglet au double clic
		if(e.getSource() == this.onglets && this.onglets.getSelectedIndex() != 0 && (e.getClickCount() == 2 || e.getButton() == MouseEvent.BUTTON2))
		{
			this.conversations.remove(this.onglets.getTitleAt(this.onglets.getSelectedIndex()));
			this.onglets.removeTabAt(this.onglets.getSelectedIndex());
		}
	}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
}
