package fr.iut.valence.java.M2M.client;

import java.awt.BorderLayout;
import java.awt.TextArea;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import fr.iut.valence.java.M2M.Emetteur;
import fr.iut.valence.java.M2M.Message;

public class OngletConversationApplicationGraphique extends JPanel implements ActionListener, KeyListener
{
	private static final long serialVersionUID = 1L;
	
	// Objets de l'interface
	private JTextField edit_message;
	private TextArea edit_conversation;
	private JButton btn_envoyer;
	private Emetteur socket;
	
	// Information sur la conversation
	private String destPseudo;
	private String pseudo;

	
	/**
	 * Constructeur de la fenêtre de conversation
	 * @param socket Objet implémentant Emetteur
	 * @param destPseudo Pseudo du destinataire
	 * @param pseudo Pseudo du client
	 * @param fenPrincipaleApplicationGraphique 
	 */
	public OngletConversationApplicationGraphique(Emetteur socket, String destPseudo, String pseudo)
	{
		this.socket = socket;
		this.destPseudo = destPseudo;
		this.pseudo = pseudo;
		
		// Initialisation des objets de l'onglet
		JPanel panelConversation = new JPanel();
		JPanel panelReponse = new JPanel();
		
		this.edit_message = new JTextField("");
		this.btn_envoyer = new JButton("Saouler");
		this.edit_conversation = new TextArea("Conversation");
		this.edit_conversation.setEditable(false);
		
		panelConversation.setLayout(new BorderLayout());
		panelConversation.add(this.edit_conversation, BorderLayout.CENTER);
		panelReponse.setLayout(new BorderLayout());
		panelReponse.add(this.edit_message, BorderLayout.CENTER);
		panelReponse.add(this.btn_envoyer, BorderLayout.EAST);
				
		this.setLayout(new BorderLayout());
		this.add(panelConversation, BorderLayout.CENTER);
		this.add(panelReponse, BorderLayout.SOUTH);

		// Ajout de la gestion des événements
		this.btn_envoyer.addActionListener(this);
		this.edit_message.addKeyListener(this);
	}
	
	/**
	 * Méthode d'affichage d'un nouveau message dans la fenêtre
	 * @param m
	 */
	public void affMessage(Message m)
	{
		this.edit_conversation.setText(this.edit_conversation.getText() + "\n" + m.getSource() + " > " + m.getTexte());
		
		// Scroll tout en bas
		this.edit_conversation.setSelectionStart(this.edit_conversation.toString().length()-1);
		this.edit_conversation.setSelectionEnd(this.edit_conversation.toString().length());
	}
	
	/**
	 * Méthode d'envoi du message saisi
	 */
	private void envoiMessage()
	{
		Message m = new Message(this.destPseudo, this.pseudo, this.edit_message.getText());
		try
		{
			this.socket.envoiMessage(m);
			this.affMessage(m);
		}
		catch (IOException e1)
		{
			JOptionPane.showMessageDialog(null, "Erreur lors de l'envoi. Réessayez.");
		}
	}

	/**
	 *	Gestion des événements
	 */
	public void actionPerformed(ActionEvent e)
	{
		// Clic sur le bouton envoyer
		if(e.getSource() == this.btn_envoyer)
		{
			if(this.edit_message.getText().length() > 0)
			{
				this.envoiMessage();
				this.edit_message.setText("");
			}
		}
	}

	/**
	 * Gestion des événements clavier
	 */
	public void keyPressed(KeyEvent e) {
		// Appuis sur entrée
		if(e.getKeyCode() == KeyEvent.VK_ENTER)
		{
			if(this.edit_message.getText().length() > 0)
			{
				this.envoiMessage();
				this.edit_message.setText("");
			}
		}
	}

	public void keyReleased(KeyEvent arg0) {};
	public void keyTyped(KeyEvent arg0) {};
}
