package fr.iut.valence.java.M2M.client;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.InetSocketAddress;
import java.net.Socket;

import fr.iut.valence.java.M2M.Emetteur;
import fr.iut.valence.java.M2M.Erreur;
import fr.iut.valence.java.M2M.Message;
import fr.iut.valence.java.M2M.Syntaxe;

/**
 * Classe de communication via le réseau
 **/
public class SocketCommunicationClient extends Thread implements Emetteur
{
	private ApplicationClient appClient;
	private Syntaxe syntaxe;
	
	private Socket socket;
	// Flux en sortie
	private OutputStreamWriter osw;
	private InfoClient infClient;

	/**
	 * Constructeur
	 * @param iClient 
	 * @param InetSocketAddress Adresse du serveur
	 * @param ApplicationClient application cible
	 * @param Syntaxe protocole de communication
	 * @throws IOException 
	 * @throws UnsupportedEncodingException 
	 */
	public SocketCommunicationClient(InetSocketAddress serveurAdresse, Syntaxe syntaxe, InfoClient infClient) throws UnsupportedEncodingException, IOException
	{
		super();
		
		this.infClient = infClient;
		this.syntaxe = syntaxe;
		
		// Connexion au serveur
		this.socket = new Socket();
		this.socket.connect(serveurAdresse, 2000);
		
		// Initialisation du flux en écriture
		this.osw = new OutputStreamWriter(this.socket.getOutputStream(), "UTF-8");
	}


	/**
	 * Méthode d'envoie d'un message via le réseau
	 * @param Message le message à envoyer
	 * @throws IOException
	 */
	public void envoiMessage(Message msg) throws IOException
	{
		this.osw.write(this.syntaxe.cmdMessage(msg) + "\n");
		this.osw.flush();
	}
	
	/**
	 * Thread de scrutation
	 */
	public void run()
	{
		BufferedReader buffer = null;
		try
		{
			// Initialisation du flux en lecture
			buffer = new BufferedReader(new InputStreamReader(this.socket.getInputStream(), "UTF-8"));
			
			// Identification sur le serveur
			this.osw.write(this.syntaxe.cmdPseudo(this.infClient.getPseudo()) + "\n");
			this.osw.flush();
		}
		catch (Exception e1)
		{
			this.appClient.receptionErreur(new Erreur(Erreur.CONNEXION, "Erreur de communication avec le serveur."));
		}

		
		// Réception des lignes de commandes
		while(true)
		{
			String line;		

			// Réception d'une ligne
			try
			{
				line = buffer.readLine();

				// Interprêtation de la commande
				Message msg = this.syntaxe.getMessage(line);
				Erreur erreur = this.syntaxe.getErreur(line);
				
				// Traitement du message
				if (msg != null)
				{
					this.appClient.receptionMessage(msg);
				}
				// Traitement d'une erreur
				else if(erreur != null)
				{
					this.appClient.receptionErreur(erreur);
				}
			}
			catch (IOException e)
			{
				this.appClient.receptionErreur(new Erreur(Erreur.RECEPTION_MESSAGE, "Erreur lors de la réception d'un message."));
			}
		}
	}

	/**
	 * Définition de l'application client
	 * @param appClient Application Client
	 */
	public void setAppClient(ApplicationClient appClient)
	{
		this.appClient = appClient;
	}
}
