package fr.iut.valence.java.M2M.serveur;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.Socket;

import fr.iut.valence.java.M2M.Emetteur;
import fr.iut.valence.java.M2M.Erreur;
import fr.iut.valence.java.M2M.Message;
import fr.iut.valence.java.M2M.Syntaxe;

/**
 * Classe de représentation d'un client par le serveur
 */
public class SocketClient extends Thread
{
	Socket s;
	Syntaxe syntaxe;
	
	// Flux en écriture
	OutputStream os = null;
	OutputStreamWriter osw = null;

	private Emetteur emetteur;
	private String pseudo;
	private SocketCommunicationServeur sockClient;
	
	
	/**
	 * Constructeur de l'objet SocketClient
	 * @param sock connexion avec le client
	 * @param sockClient
	 * @param syntaxe
	 * @param emetteur objet envoyant les messages
	 * @param pseudo
	 * @throws IOException
	 */
	public SocketClient(Socket sock, SocketCommunicationServeur sockClient, Syntaxe syntaxe, Emetteur emetteur, String pseudo) throws IOException
	{
		this.sockClient = sockClient;
		this.pseudo = pseudo;
		this.emetteur = emetteur;
		this.s = sock;
		this.syntaxe = syntaxe;
		
		// Initialisation du flux en sortie
		this.os = this.s.getOutputStream();
		this.osw = new OutputStreamWriter(this.os, "UTF-8");
	}
	
	
	/**
	 * Envoie un message au client
	 * @param Message le message à envoyer
	 */
	public void receptionMessage(Message msg)
	{
		try
		{
			this.osw.write(this.syntaxe.cmdMessage(msg) + "\n");
			this.osw.flush();
		}
		catch (IOException e)
		{
			System.out.println("Erreur d'envoi du message à " + this.pseudo);
		}
	}
	
	/**
	 * Envoi d'un erreur au client
	 * @param err
	 */
	public void receptionErreur(Erreur err)
	{
		try
		{
			this.osw.write(this.syntaxe.cmdErreur(err) + "\n");
			this.osw.flush();
		}
		catch (IOException e)
		{
			System.out.println("Erreur de l'envoi de l'erreur à " + this.pseudo);
		}
	}
	
	/**
	 * Thread de scrutation
	 */
	public void run()
	{
		// Initialisation du flux en lecture
		InputStream i;
		InputStreamReader ir;
		BufferedReader br = null;
		try
		{
			i = this.s.getInputStream();
			ir = new InputStreamReader(i, "UTF-8");
			br = new BufferedReader(ir);
		}
		catch (IOException e)
		{
			System.out.println("Connexion avec " + this.pseudo + " terminée.");
			this.sockClient.suppressionClient(this.pseudo);
			return;
		}
				
		// Réception des lignes de commandes
		while(true)
		{
			String line;

			// Réception d'une ligne
			try
			{
				// Réception d'une commande
				line = br.readLine();
				if(line != null) {
					// Réception d'un message
					Message mess = this.syntaxe.getMessage(line);
					
					// Traitement du message
					if (mess != null)
					{
						this.emetteur.envoiMessage(mess);
					}
				}
				// Le client a terminé la connexion
				else
				{
					throw(new IOException());
				}
			}
			// Terminaison de la connexion
			catch (IOException e)
			{
				System.out.println("Connexion avec " + this.pseudo + " terminée.");
				this.sockClient.suppressionClient(this.pseudo);
				break;
			}
		}
	}
}
