package fr.iut.valence.java.M2M.serveur;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.SortedMap;
import java.util.TreeMap;

import fr.iut.valence.java.M2M.Emetteur;
import fr.iut.valence.java.M2M.Erreur;
import fr.iut.valence.java.M2M.Message;
import fr.iut.valence.java.M2M.Syntaxe;

public class SocketCommunicationServeur extends Thread implements Emetteur
{
	// Collection contenant les clients connectés
	private SortedMap<String, SocketClient> clients;
	private Syntaxe syntaxe;
	private InetSocketAddress sa;
	
	/**	
	 * Constructeur
	 */
	public SocketCommunicationServeur(Syntaxe syntaxe, InetSocketAddress serveurAdresse)
	{
		this.syntaxe = syntaxe;
		this.sa = serveurAdresse;
		this.clients = new TreeMap<String, SocketClient>();
	}
	
	/**
	 * Méthode d'ajout d'un nouveau client
	 * @param cli le client à ajouter
	 * @throws IOException 
	 */
	public void ajoutClient(String pseudo, SocketClient cli) throws IOException
	{
		this.clients.put(pseudo, cli);
	}

	/**
	 * Suppression d'un client
	 * @param pseudo
	 */
	public void suppressionClient(String pseudo)
	{
		this.clients.remove(pseudo);
	}
	
	/**
	 * Transmission d'un message
	 * @param msg
	 */
	public void envoiMessage(Message msg) throws IOException
	{
		if(this.clients.containsKey(msg.getDestinataire()))
		{
			this.clients.get(msg.getDestinataire()).receptionMessage(msg);
		}
		else
		{
			this.envoiErreur(msg.getSource(), new Erreur(Erreur.PSEUDO_INEXISTANT,"Le pseudo demandé n'est pas attribué."));
		}
	}
	/**
	 * Envoie d'une erreur à un client
	 * @param String destinataire 
	 * @param String Erreur
	 */
	private void envoiErreur(String source, Erreur err)
	{	
		if(this.clients.containsKey(source))
		{
			this.clients.get(source).receptionErreur(err);
		}	
	}

	/*
	 * Methode de bienvenue
	 * Crée les threads SocketClient
	 */
	public void run()
	{
		ServerSocket ss = null;
		
		System.out.println("Starting...");
		try
		{
			ss = new ServerSocket();
			ss.bind(this.sa);
		}
		catch (IOException e)
		{
			System.out.println("Impossible d'ouvrir le port en scrutation. Vérifier que vous disposez des droits nécessaires.");
			return;
		}
		
		System.out.println("Binding...");
		while (true)
		{
			Socket s;
			try
			{
				s = ss.accept();
				
				System.out.println("Connexion depuis "+ s.getRemoteSocketAddress());
				
				// Identification du client
				BufferedReader br = new BufferedReader(new InputStreamReader(s.getInputStream()));
				String pseudo = this.syntaxe.getPseudo(br.readLine());
				
				SocketClient cli = new SocketClient(s, this, this.syntaxe, this, pseudo);
				
				// Pseudonyme déjà utilisé
				if(pseudo != null && this.clients.containsKey(pseudo))
				{
					System.out.println(pseudo + " déja utilisé.");
					OutputStreamWriter osw = new OutputStreamWriter(s.getOutputStream(), "UTF-8");
					osw.write(this.syntaxe.cmdErreur(new Erreur(Erreur.PSEUDO_DEJA_UTILISE, pseudo+" : pseudonyme déjà utilisé")) + "\n");
					osw.flush();
				}
				// Ajout du client, et lancement du thread associé
				else if(pseudo != null)
				{
					System.out.println(pseudo + " connecté.");
					cli.start();
					this.ajoutClient(pseudo, cli);
				}
			}
			catch (IOException e)
			{
				System.out.println("Erreur de connexion avec le client.");
			}
		}
	}	
}
